from upyog.imports import *


"""
Taken from fastaiv1: https://github.com/fastai/fastai1/blob/a8327427ad5137c4899a1b4f74745193c9ea5be3/fastai/data_block.py#L22-L47
"""

__all__ = ["PathLike", "get_files", "get_image_files", "get_video_files"]


def get_files(
    path: PathLike,
    extensions: Collection[str] = None,
    recurse: bool = False,
    exclude: Optional[Collection[str]] = None,
    include: Optional[Collection[str]] = None,
    presort: bool = False,
    followlinks: bool = False,
) -> List[Path]:
    """
    Return list of files in `path` that have a suffix in `extensions`; optionally `recurse`.
    Use `include` and `exclude` for including/excluding folder names, `presort` to sort.
    """
    # fmt: off
    if recurse:
        res = []
        for i,(p,d,f) in enumerate(os.walk(path, followlinks=followlinks)):
            # skip hidden dirs
            if include is not None and i==0:   d[:] = [o for o in d if o in include]
            elif exclude is not None and i==0: d[:] = [o for o in d if o not in exclude]
            else:                              d[:] = [o for o in d if not o.startswith('.')]
            res += _get_files(path, p, f, extensions)
        if presort: res = sorted(res, key=lambda p: _path_to_same_str(p), reverse=False)
        return res
    else:
        f = [o.name for o in os.scandir(path) if o.is_file()]
        res = _get_files(path, path, f, extensions)
        if presort: res = sorted(res, key=lambda p: _path_to_same_str(p), reverse=False)
        return res
    # fmt: on


def _path_to_same_str(p_fn: PathLike) -> str:
    "path -> str, but same on nt+posix, for alpha-sort only"
    s_fn = str(p_fn)
    s_fn = s_fn.replace("\\", ".")
    s_fn = s_fn.replace("/", ".")
    return s_fn


def _get_files(parent, p, f, extensions) -> list:
    p = Path(p)  # .relative_to(parent)
    if isinstance(extensions, str):
        extensions = [extensions]
    low_extensions = [e.lower() for e in extensions] if extensions is not None else None
    res = [
        p / o
        for o in f
        if not o.startswith(".")
        and (extensions is None or f'.{o.split(".")[-1].lower()}' in low_extensions)
    ]
    return res


image_extensions = set(
    k for k, v in mimetypes.types_map.items() if v.startswith("image/")
)
video_extensions = set(
    [k for k, v in mimetypes.types_map.items() if v.startswith("video/")] + [".mkv"]
)


def get_image_files(
    path: PathLike,
    include: Optional[List[str]] = None,
    exclude: Optional[List[str]] = None,
    recurse: bool = True,
) -> List[Path]:
    return get_files(
        path=path,
        include=include,
        exclude=exclude,
        recurse=recurse,
        extensions=image_extensions,
    )


def get_video_files(
    path: PathLike,
    include: Optional[List[str]] = None,
    exclude: Optional[List[str]] = None,
    recurse: bool = True,
) -> List[Path]:
    return get_files(
        path=path,
        include=include,
        exclude=exclude,
        recurse=recurse,
        extensions=video_extensions,
    )



"""
This module makes some adjustments to the default behavior of the excellent
`fastcore.script` package to easily create functions that can be easily used
in regular code _and_ for command line interfaces with minimal additional code
"""

from upyog.imports import *
from fastcore.all import Param

__all__ = ["Param", "call_parse"]


class Param(Param):
    def __init__(
        self,
        help=None,
        type=None,
        opt=True,
        action=None,
        nargs=None,
        const=None,
        choices=None,
        required=None,
        default=None,
        metavar="",
    ):
        if type == fastcore.store_true or action == "store_true":
            type, action, default, metavar = None, "store_true", False, None
        if type == fastcore.store_false or action == "store_false":
            type, action, default, metavar = None, "store_false", True, None
        if (
            type
            and isinstance(type, typing.Type)
            and issubclass(type, enum.Enum)
            and not choices
        ):
            choices = list(type)
        fastcore.store_attr()


SCRIPT_INFO = SimpleNamespace(func=None)


class Formatter(argparse.RawDescriptionHelpFormatter, argparse.HelpFormatter):
    ...


def anno_parser(func, prog=None, from_name=False):
    "Look at params (annotated with `Param`) in func and return an `ArgumentParser`"
    cols = shutil.get_terminal_size((120, 30))[0]
    fmtr = partial(Formatter, max_help_position=cols // 2, width=cols)
    p = argparse.ArgumentParser(
        description=func.__doc__, prog=prog, formatter_class=fmtr
    )
    for k, v in inspect.signature(func).parameters.items():
        param = func.__annotations__.get(k, fastcore.Param())
        param.set_default(v.default)
        p.add_argument(f"{param.pre}{k}", **param.kwargs)
    p.add_argument(f"--pdb", help=argparse.SUPPRESS, action="store_true")
    # p.add_argument(f"--xtra", help=argparse.SUPPRESS, type=str)
    return p


def call_parse(func):
    "Decorator to create a simple CLI from `func` using `anno_parser`"
    mod = inspect.getmodule(inspect.currentframe().f_back)
    if not mod:
        return func

    @functools.wraps(func)
    def _f(*args, **kwargs):
        mod = inspect.getmodule(inspect.currentframe().f_back)
        if not mod:
            return func(*args, **kwargs)
        if not SCRIPT_INFO.func and mod.__name__ == "__main__":
            SCRIPT_INFO.func = func.__name__
        if len(sys.argv) > 1 and sys.argv[1] == "":
            sys.argv.pop(1)
        p = anno_parser(func)
        args = p.parse_args().__dict__
        xtra = fastcore.otherwise(args.pop("xtra", ""), fastcore.eq(1), p.prog)
        tfunc = fastcore.trace(func) if args.pop("pdb", False) else func
        tfunc(**fastcore.merge(args, fastcore.args_from_prog(func, xtra)))

    if mod.__name__ == "__main__":
        setattr(mod, func.__name__, _f)
        SCRIPT_INFO.func = func.__name__
        return _f()
    else:
        return _f



from upyog.imports import *
from upyog.os.read_files import *
from upyog.utils import *
from upyog.cli import Param as P, call_parse


__all__ = ["move_files", "cleanup_duplicates"]


@call_parse
def move_files(
    i: P("Input folders to move the files from", str, nargs="+", metavar="INP") = None,
    o: P("Path to the output folders", str) = None,
    move: P("Move files (desctructively)", action="store_true") = False,
):
    """
            --------- FILE MOVER ---------

    Move files from an arbitrary number of input folders to a _single_ output folder.

    The input folders are scanned recursively.
    If the output folder doesn't exist, it is created
    """
    recurse = True
    files = flatten([get_files(p, recurse=recurse) for p in i])
    out_dir = Path(o)
    if not out_dir.exists():
        rich.print(f"Creating output folder: {out_dir}")
        out_dir.mkdir(exist_ok=True, parents=True)

    mover = shutil.move if move else shutil.copy

    for file in track(files, "Moving Files..."):
        mover(str(file), str(out_dir))


@call_parse
def cleanup_duplicates(
    i: P("Single input folder that is to be cleaned up", str) = None,
    t: P(
        "An arbitrary number of target folders that contain duplicate files that are to be removed from the input folder `--i`",
        str,
        nargs="+",
    ) = None,
):
    """
            --------- DUPLICATE REMOVER ---------

    This program scans the _names_ of the files in the input and target folders
    recursively, and removes all the files from the `input` folder that exist in
    `target` folders

    If input folder has the following files:
    ├── input-folder
    │   ├── file1.jpg
    │   ├── file2.jpg
    │   └── file3.jpg

    And the target folder(s) have the following files:
    ├── target-folder-1
    │   ├── file1.jpg
    │   ├── other-random-file-1.jpg

    ├── target-folder-2
    │   ├── file2.jpg
    │   ├── other-random-file-2.jpg

    Using this program would remove `file1.jpg` and `file2.jpg` from the input folders

    Usage:
    ------

    remove-duplicates-from-folder
        --i input-folder
        --t target-folder-1   target-folder-2   ...
    """
    cleanup_path = Path(i)
    target_paths = [Path(p) for p in t]

    target_fnames = flatten([get_image_files(path) for path in target_paths])
    target_fnames = [f.stem for f in target_fnames]

    cleanup_files = get_image_files(cleanup_path)
    cleanup_fnames = [f.stem for f in cleanup_files]

    fnames_to_delete = set.intersection(set(cleanup_fnames), set(target_fnames))

    counter = 0
    for f in track(cleanup_files, "Scanning Files..."):
        if f.stem in fnames_to_delete:
            print(f"Deleting {f.name}")
            counter += 1
            f.unlink()

    print(f"Deleted {counter} files")



from upyog.imports import *


__all__ = ["flatten", "uniqueify"]


def flatten(x: Any) -> List[Any]:
    flattened_list = []
    for item in x:
        if isinstance(item, (tuple, list)):
            [flattened_list.append(i) for i in item]
        else:
            flattened_list.append(item)
    return flattened_list


def uniqueify(x: Collection) -> Collection:
    return sorted(list(set(x)))



from turtle import pos
from upyog.imports import *
from upyog.image.draw import *
from upyog.image.composition import *
from upyog.image.draw import TEXT_POSITIONS


__all__ = ["Visualiser"]


class Visualiser:
    def __init__(self, img: Image.Image, font_path: Optional[str] = None, font_size=30):
        copy = True
        self.img = deepcopy(img) if copy else img
        self.base_font_size = font_size

        self.font_path = font_path
        if not font_path:
            self.font_path = str(
                Path(__file__).parent.parent.parent / "assets" / "EuroStyleNormal.ttf"
            )

    def _repr_png_(self):
        return self.img._repr_png_()

    def __repr__(self):
        return f"Visualiser(img={self.img}, font_path={self.font_path}"

    @property
    def size(self):
        return self.img.size

    def draw_rectangle(
        self,
        xyxy: tuple,
        fill: Optional[tuple] = (255, 255, 255),
        opacity=0.25,
        width=None,
    ):
        self.img = draw_rectangle(self.img, xyxy, fill, opacity, width)
        return self.img

    def draw_rectangles(
        self,
        xyxys: List[tuple],
        fill: Optional[tuple] = (255, 255, 255),
        opacity=0.25,
        width=None,
    ):
        for xyxy in xyxys:
            self.img = self.draw_rectangle(xyxy, fill, opacity, width)
        return self.img

    def draw_ellipse(self, xyxy, fill=(255, 255, 255), opacity=0.8):
        self.img = draw_ellipse(self.img, xyxy, fill, opacity)
        return self.img

    def draw_circle(self, xy, radius, fill=(255, 255, 255), opacity=0.8):
        self.img = draw_circle(self.img, xy, radius, fill, opacity)
        return self.img

    def draw_keypoints(
        self,
        xys: List[Tuple[int, int]],
        fill=(255, 255, 255),
        opacity=0.8,
        radius=None,
        dynamic_radius=True,
    ):
        self.img = draw_keypoints(self.img, xys, fill, opacity, radius, dynamic_radius)
        return self.img

    def draw_rounded_rectangle(
        self, xyxy: tuple, radius=10, color="white", opacity=1.0, width=3
    ):
        self.img = draw_rounded_rectangle(self.img, xyxy, radius, color, opacity, width)
        return self.img

    def draw_rounded_rectangles(
        self, xyxys: List[tuple], radius=10, color="white", opacity=1.0
    ):
        for xyxy in xyxys:
            self.img = self.draw_rounded_rectangle(xyxy, radius, color, opacity)
        return self.img

    def draw_vertical_bars(
        self,
        width_percentages: List[Union[float, tuple]],
        fill=(255, 255, 255),
        thickness=5,
        opacity=0.4,
    ):
        self.img = draw_vertical_bars(
            self.img, width_percentages, fill, thickness, opacity
        )
        return self.img

    def draw_horizontal_bars(
        self,
        height_percentages: List[Union[float, tuple]],
        fill=(255, 255, 255),
        thickness=5,
        opacity=0.4,
    ):
        self.img = draw_horizontal_bars(
            self.img, height_percentages, fill, thickness, opacity
        )
        return self.img

    def draw_bbox(
        self,
        xyxy,
        label: str,
        label_transform: Callable = str.upper,
        label_location: Literal["top", "bottom"] = "bottom",
        confidence: Optional[float] = None,
        style: Literal["rounded", "sharp"] = "rounded",
        corner_radius=10,  # Only if `style` == "rounded"
        box_fill=(255, 255, 255),
        font_fill=(255, 255, 255),
        font_bordered=False,
        font_background=True,
        font_background_opacity=0.15,
        font_pad=0.03,
        border_opacity=1.0,
        border_width=3,
    ):

        if style == "sharp":
            self.img = self.draw_rectangle(xyxy, box_fill, border_opacity, border_width)
        elif style == "rounded":
            self.img = self.draw_rounded_rectangle(
                xyxy, corner_radius, box_fill, border_opacity, border_width
            )

        label = label_transform(label)
        if confidence:
            label = f"{label}: {round(confidence*100, 1)}%"

        self.img = self.draw_text_within_xyxy(
            xyxy,
            label,
            font_pad,
            font_fill,
            label_location,
            font_bordered,
            font_background,
            font_background_opacity,
        )

        return self.img

    def draw_text_within_xyxy(
        self,
        xyxy,
        label,
        font_pad,
        font_fill,
        label_location,
        font_bordered=False,
        font_bg=True,
        font_bg_opacity=0.15,
    ):
        self.img = draw_text_within_xyxy(
            img=self.img,
            xyxy=xyxy,
            label=label,
            font_path=self.font_path,
            base_font_size=self.base_font_size,
            pad_percentage=font_pad,
            text_color=font_fill,
            location=label_location,
            bordered=font_bordered,
            add_background=font_bg,
            background_opacity=font_bg_opacity,
        )
        return self.img

    def draw_text(
        self,
        text: Union[str, List[str]],
        position: TEXT_POSITIONS,
        # font_path: Optional[PathLike] = None,
        # font_size: int = 30,
        font_border=False,
        font_color=(255, 255, 255),
        font_border_color=(0, 0, 0),
        padding: Union[int, float] = 0.03,
        line_spacing: float = 1.3,  # Only used if text is List[str]
        height_constraints: Tuple[int, int] = (0.0, 1.0),
        width_constraints: Tuple[int, int] = (0.0, 1.0),
    ):
        return draw_text(
            img=self.img,
            text=text,
            position=position,
            font_path=self.font_path,
            font_size=self.base_font_size,
            font_border=font_border,
            font_color=font_color,
            font_border_color=font_border_color,
            padding=padding,
            line_spacing=line_spacing,
            height_constraints=height_constraints,
            width_constraints=width_constraints,
        )

    def caption(self, text: str, color=(255, 255, 255)):
        xyxy = (0, 0, self.img.width, self.img.height)
        self.img = self.draw_text_within_xyxy(
            xyxy, text, 0.05, color, "bottom", font_bg=False, font_bordered=True
        )
        return self.img

    def draw_rule_of_thirds(self, thickness=5, opacity=0.4):
        return draw_rule_of_thirds(self.img, thickness, opacity)
